//Copyright 1999-2002 Gavin Conant

#include <iostream.h>
#include <iomanip.h>
#include <string.h>
#include "gen_dna_funcs.h"
#include "read_seq.h"
#include "pair_dist.h"



int main (int argc, char **argv)
{
  int i, j, nchars, new_nchars, ntaxa, cnt=0, aligned_res=0;
  char infile[100];
  BOOL protein_seq=FALSE, is_gap, no_gaps=FALSE, first=TRUE, found_gap=FALSE;
  char (*convert_to_nuc_aa_code)(int);
  double mean_syn=0, mean_non_syn=0; 
  Read_Sequence *read_seq;
  Sequence_dataset *current_data, *new_dataset;
  Exchange current_exchange;
  Pair_dist *percent_dist;
 if (argc<3)
   {
     cerr<<"Usage: calc_percent_diff <infile> -d:(dna/protein) (-nogaps)\n";
     return(-1);
   }
 else
   {
     strcpy(infile, argv[1]);
     
     if (argc==4)
       no_gaps=TRUE;

     switch(guess_dataformat(infile, strlen(infile)))
       {
       case NEXUS:
	 read_seq=new Read_Nexus;
	 break;
       case PIR:
	 read_seq=new Read_PIR;
	 break;
       case PHYLIP:
	 read_seq=new Read_Phylip_interleave;
	 break;
       case FASTA:
	 read_seq=new Read_FASTA;
	 break;
       } 

     if ((argv[2][3] == 'P') || (argv[2][3] == 'p'))
       protein_seq=TRUE;
	 
     current_data=read_seq->get_dataset(ntaxa, nchars, infile, protein_seq);   
     current_exchange.set_num_taxa(ntaxa);

     if (protein_seq == FALSE)
       convert_to_nuc_aa_code=&num_to_base;
	   
     else
	 convert_to_nuc_aa_code=&num_to_aa;
	

     for (i=0; i<nchars; i++)
       {
	 is_gap=FALSE;
	 for(j=0; j<ntaxa; j++)   
	   if (convert_to_nuc_aa_code((*current_data)[j][i]) == '-')
	     is_gap=TRUE;
	 if (is_gap == FALSE) aligned_res++;
       }

     if (no_gaps == TRUE)
       {
	 new_dataset= remove_gaps (current_data, protein_seq);
	 delete current_data;
	 current_data=new_dataset;
	 nchars=(*current_data)[0].Sequence_size();
	 current_exchange.set_num_sites(nchars);
       }

	 
     if (protein_seq == FALSE)
       percent_dist=new Percent_dist(current_data, &current_exchange);
      
     else
       percent_dist=new AA_Percent_dist(current_data, &current_exchange);
      
     percent_dist->find_dist();
     
     cout<<"Number of non-gap aligned residues: "<<aligned_res<<endl;
     cout<<"Percent pair-wise difference for current data: \n";
     percent_dist->print_dists();
     cout<<endl;
     
   
     delete percent_dist;
     delete read_seq;
     delete current_data;
     return(0);
   }
}//end main



